// Fragment Shader  file "lambert.frag"

#version 330 core

// Material uniforms

uniform vec3 colorOverlay;

uniform float ambientFactor;
uniform sampler2D diffuseTexture;
uniform float diffuseFactor;

uniform float specularFactor;
uniform float shininess;
uniform sampler2D specularMap;
uniform int useSpecularMap;

uniform vec3 ambientColor;

// Light uniforms

uniform vec3 spotLightDirection[8];
uniform float spotLightCosInner[8];
uniform float spotLightCosOuter[8];

uniform vec3 omniLightPosition[8];
uniform vec3 omniLightColor[8];
uniform float omniLightLinAtt[8];
uniform float omniLightQuadrAtt[8];
uniform float omniLightConstAtt[8];
uniform int numOmniLights;

// Coordinates

in vec2 ex_coord;
in vec4 mvPosition;
in vec3 mvNormal;
out vec4 out_color;

void main(void)
{
	vec4 tex = texture(diffuseTexture, ex_coord);
	vec3 totalColor = ambientFactor * ambientColor;

	vec3 n = mvNormal;

	vec3 totalSpecular = vec3(0.0,0.0,0.0);

	float specularMultiplier = specularFactor;

	if (useSpecularMap==1)
	{
		specularMultiplier *= texture(specularMap, ex_coord).r;
	}
	
	float sum=0.0;
	for(int i=0; i<numOmniLights; i++)
	{
		// diffuse component
		vec3 ray =  (omniLightPosition[i] - mvPosition.xyz);
		float dist = length(ray);
		ray = ray;
		vec3 l = normalize(ray);

		float spotFactor;
		if (spotLightCosInner[i] < 1.0)
		{
			vec3 lDir = normalize(-spotLightDirection[i]);
			float cosSpot = dot(l, lDir);
			if (cosSpot > spotLightCosInner[i])
			{
				spotFactor = 1.0;
			}
			else if (cosSpot < spotLightCosOuter[i])
			{
				spotFactor = 0.0;
			}
			else
			{
				spotFactor = 1.0 - (spotLightCosInner[i] - cosSpot) / (spotLightCosInner[i] - spotLightCosOuter[i]);
			}
			sum+=spotFactor;
		}
		else
		{
			spotFactor = 1.0;
		}

		float dotProd = spotFactor * max(dot(l, n), 0);
		vec3 difLight = dotProd * omniLightColor[i];
		float att = omniLightConstAtt[i] + omniLightLinAtt[i] * dist + omniLightQuadrAtt[i] * dist * dist;
		totalColor += diffuseFactor*(difLight/att);

		// specular light
		if (specularFactor > 0.0)
		{
			vec3 eye = normalize(mvPosition.xyz);
			vec3 r = reflect(l, n);

			float cosAlpha = clamp(dot(eye, r), 0, 1);
			totalSpecular += dotProd * omniLightColor[i] * specularMultiplier * (pow(cosAlpha, shininess) / att);
		}
	}

	out_color = vec4(colorOverlay,0.0) + vec4(totalColor * tex.xyz + totalSpecular, tex.a);
	//out_color = vec4(1.0,1.0,1.0,tex.a);
}