// Vertex Shader  file "lambert.vert"

#version 330 core

uniform int hasBones;

uniform mat4 mv;
uniform mat4 perspective;
uniform mat4 normalMatrix;

uniform mat4 boneTransform[50];
uniform mat4 boneNormalTransform[50];

layout(location=1) in  vec3 position;
layout(location=2) in  vec3 normal;
layout(location=5) in  vec2 tex_coord;

layout(location=10) in  ivec2 bone_influence;
layout(location=11) in  vec2 bone_weight;

out vec2 ex_coord;
out vec4 mvPosition;
out vec3 mvNormal;


void main(void)
{
	vec4 p;
	vec4 n;

	if (hasBones==1)
	{
		vec4 p1 = (boneTransform[bone_influence.x] * vec4(position,1.0)) * bone_weight.x;
		p = p1+(boneTransform[bone_influence.y] * vec4(position,1.0)) * bone_weight.y;

		vec4 n1 = (boneNormalTransform[bone_influence.x] * vec4(normal,1.0)) * bone_weight.x;
		n = n1+(boneNormalTransform[bone_influence.y] * vec4(normal,1.0)) * bone_weight.y;
	} else {
		p = vec4(position,1.0);
		n = vec4(normal,1.0);
	}

	mvPosition = mv * p;
	gl_Position = perspective * mvPosition;

	mvNormal = normalize((normalMatrix * n).xyz);

	ex_coord = tex_coord;

}